@@ -313,6 +313,7 @@ _gdk_windowing_window_init (void)
   private->window_type = GDK_WINDOW_ROOT;
   private->state       = 0;
   private->children    = NULL;
+  private->viewable = TRUE;
 //  impl->drawable.paint_region   = NULL;
   impl->gdkWindow      = _gdk_parent_root;
   impl->window           = NULL;
@@ -350,7 +351,6 @@ _gdk_windowing_window_init (void)
 }
 
 
-
 GdkWindow *
 gdk_directfb_window_new (GdkWindow              *parent,
                          GdkWindowAttr          *attributes,
@@ -447,6 +447,7 @@ gdk_directfb_window_new (GdkWindow              *parent,
       desc.posy   = y;
       desc.width  = impl->drawable.width;
       desc.height = impl->drawable.height;
+
 #if 0
       if (window_caps)
         {
@@ -468,16 +469,22 @@ gdk_directfb_window_new (GdkWindow              *parent,
 
       if (!create_directfb_window (impl, &desc, window_options))
         {
-		  g_assert(0);
+          g_assert(0);
           _gdk_window_destroy (window, FALSE);
+
           return NULL;
         }
-      	if( desc.caps != DWCAPS_INPUTONLY )
-			impl->window->SetOpacity(impl->window, 0x00 );
+
+      if (desc.caps != DWCAPS_INPUTONLY)
+        {
+          impl->window->SetOpacity(impl->window, 0x00 );
+        }
+
       break;
 
     case GDK_WINDOW_CHILD:
-	   impl->window=NULL;
+      impl->window=NULL;
+
       if (!private->input_only && parent_impl->drawable.surface)
         {
 
@@ -487,12 +494,14 @@ gdk_directfb_window_new (GdkWindow              *parent,
                                                         &rect,
                                                         &impl->drawable.surface);
         }
+
       break;
 
     default:
       g_warning ("gdk_window_new: unsupported window type: %d",
                  private->window_type);
       _gdk_window_destroy (window, FALSE);
+
       return NULL;
     }
 
@@ -501,28 +510,28 @@ gdk_directfb_window_new (GdkWindow              *parent,
       GdkColormap *colormap;
 
       impl->drawable.surface->GetPixelFormat (impl->drawable.surface,
-					      &impl->drawable.format);
+                                              &impl->drawable.format);
 
-  	  private->depth = DFB_BITS_PER_PIXEL(impl->drawable.format);
+      private->depth = DFB_BITS_PER_PIXEL(impl->drawable.format);
 
       if ((attributes_mask & GDK_WA_COLORMAP) && attributes->colormap)
-	{
-	  colormap = attributes->colormap;
-	}
+        {
+          colormap = attributes->colormap;
+        }
       else
-	{
-	  if (gdk_visual_get_system () == visual)
-	    colormap = gdk_colormap_get_system ();
-	  else
-	    colormap =gdk_drawable_get_colormap (parent);
-	}
+        {
+          if (gdk_visual_get_system () == visual)
+            colormap = gdk_colormap_get_system ();
+          else
+            colormap =gdk_drawable_get_colormap (parent);
+        }
 
       gdk_drawable_set_colormap (GDK_DRAWABLE (window), colormap);
     }
   else
     {
       impl->drawable.format = ((GdkVisualDirectFB *)visual)->format;
-  	  private->depth = visual->depth;
+      private->depth = visual->depth;
     }
 
   gdk_window_set_cursor (window, ((attributes_mask & GDK_WA_CURSOR) ?
@@ -548,17 +557,174 @@ gdk_directfb_window_new (GdkWindow              *parent,
   return window;
 }
 
-GdkWindow *
-_gdk_window_new (GdkWindow     *parent,
-                 GdkWindowAttr *attributes,
-                 gint           attributes_mask)
+
+void
+_gdk_window_impl_new (GdkWindow     *window,
+                      GdkWindow     *real_parent,
+                      GdkScreen     *screen,
+                      GdkVisual     *visual,
+                      GdkEventMask   event_mask,
+                      GdkWindowAttr *attributes,
+                      gint           attributes_mask)
 {
-  g_return_val_if_fail (attributes != NULL, NULL);
+  GdkWindowObject       *private;
+  GdkWindowObject       *parent_private;
+  GdkWindowImplDirectFB *impl;
+  GdkWindowImplDirectFB *parent_impl;
+  DFBWindowDescription   desc;
 
-  D_DEBUG_AT( GDKDFB_Window, "%s( %p )\n", __FUNCTION__, parent );
+  impl = g_object_new (_gdk_window_impl_get_type (), NULL);
+  impl->drawable.wrapper = GDK_DRAWABLE (window);
+  impl->gdkWindow = window;
 
-  return gdk_directfb_window_new (parent, attributes, attributes_mask,
-                                  DWCAPS_NONE, DWOP_NONE, DSCAPS_NONE);
+  private = GDK_WINDOW_OBJECT (window);
+  private->impl = (GdkDrawable *)impl;
+
+  private->x = (attributes_mask & GDK_WA_X) ? attributes->x : 0;
+  private->y = (attributes_mask & GDK_WA_Y) ? attributes->y : 0;
+
+  parent_private = private->parent;
+  parent_impl = GDK_WINDOW_IMPL_DIRECTFB (parent_private->impl);
+
+  private->parent = parent_private;
+
+  _gdk_directfb_calc_abs (window);
+
+  impl->drawable.width  = MAX (1, attributes->width);
+  impl->drawable.height = MAX (1, attributes->height);
+
+  private->window_type = attributes->window_type;
+
+  desc.flags = 0;
+
+  switch (attributes->wclass)
+    {
+    case GDK_INPUT_OUTPUT:
+      private->input_only = FALSE;
+
+      desc.flags |= DWDESC_PIXELFORMAT;
+      desc.pixelformat = ((GdkVisualDirectFB *)visual)->format;
+
+      if (DFB_PIXELFORMAT_HAS_ALPHA (desc.pixelformat))
+        {
+          desc.flags |= DWDESC_CAPS;
+          desc.caps = DWCAPS_ALPHACHANNEL;
+        }
+
+      break;
+
+    case GDK_INPUT_ONLY:
+      private->input_only = TRUE;
+      desc.flags |= DWDESC_CAPS;
+      desc.caps = DWCAPS_INPUTONLY;
+      break;
+
+    default:
+      g_warning ("_gdk_window_impl_new: unsupported window class\n");
+      _gdk_window_destroy (window, FALSE);
+      return;
+    }
+
+  switch (private->window_type)
+    {
+    case GDK_WINDOW_TOPLEVEL:
+    case GDK_WINDOW_DIALOG:
+    case GDK_WINDOW_TEMP:
+      desc.flags |= (DWDESC_WIDTH | DWDESC_HEIGHT |
+                     DWDESC_POSX  | DWDESC_POSY);
+      desc.posx   = private->x;
+      desc.posy   = private->y;
+      desc.width  = impl->drawable.width;
+      desc.height = impl->drawable.height;
+
+      if (!create_directfb_window (impl, &desc, DWOP_NONE))
+        {
+          g_assert (0);
+          _gdk_window_destroy (window, FALSE);
+
+          return;
+        }
+
+      if (desc.caps != DWCAPS_INPUTONLY)
+        {
+          impl->window->SetOpacity (impl->window, 0x00);
+        }
+
+      break;
+
+    case GDK_WINDOW_CHILD:
+      impl->window = NULL;
+
+      if (!private->input_only && parent_impl->drawable.surface)
+        {
+          DFBRectangle rect = { private->x,
+                                private->y,
+                                impl->drawable.width,
+                                impl->drawable.height };
+          parent_impl->drawable.surface->GetSubSurface (parent_impl->drawable.surface,
+                                                        &rect,
+                                                        &impl->drawable.surface);
+        }
+
+      break;
+
+    default:
+      g_warning ("_gdk_window_impl_new: unsupported window type: %d",
+                 private->window_type);
+      _gdk_window_destroy (window, FALSE);
+
+      return;
+    }
+
+  if (impl->drawable.surface)
+    {
+      GdkColormap *colormap;
+
+      impl->drawable.surface->GetPixelFormat (impl->drawable.surface,
+                                              &impl->drawable.format);
+
+      private->depth = DFB_BITS_PER_PIXEL (impl->drawable.format);
+
+      if ((attributes_mask & GDK_WA_COLORMAP) && attributes->colormap)
+        {
+          colormap = attributes->colormap;
+        }
+      else
+        {
+          if (gdk_visual_get_system () == visual)
+            colormap = gdk_colormap_get_system ();
+          else
+            colormap = gdk_colormap_new (visual, FALSE);
+        }
+
+      gdk_drawable_set_colormap (GDK_DRAWABLE (window), colormap);
+    }
+  else
+    {
+      impl->drawable.format = ((GdkVisualDirectFB *)visual)->format;
+      private->depth = visual->depth;
+    }
+
+  gdk_window_set_cursor (window,
+                         ((attributes_mask & GDK_WA_CURSOR) ?
+                          (attributes->cursor) : NULL));
+
+  if (parent_private)
+    parent_private->children = g_list_prepend (parent_private->children,
+                                               window);
+
+  /* we hold a reference count on ourself */
+  g_object_ref (window);
+
+  if (impl->window)
+    {
+      impl->window->GetID (impl->window, &impl->dfb_id);
+      gdk_directfb_window_id_table_insert (impl->dfb_id, window);
+      gdk_directfb_event_windows_add (window);
+    }
+
+  if (attributes_mask & GDK_WA_TYPE_HINT)
+    gdk_window_set_type_hint (window, attributes->type_hint);
 }
 
 void
@@ -1998,6 +2164,7 @@ gdk_directfb_window_input_shape_combine_region (GdkWindow       *window,
 
 static void
 gdk_directfb_window_queue_translation (GdkWindow *window,
+				       GdkGC     *gc,
                                        GdkRegion *region,
                                        gint       dx,
                                        gint       dy)
@@ -2501,6 +2668,7 @@ gdk_window_foreign_new_for_display (GdkDisplay* display,GdkNativeWindow anid)
     private->impl = g_object_new (_gdk_window_impl_get_type (), NULL);
     private->parent = parent_private;
     private->window_type = GDK_WINDOW_TOPLEVEL;
+    private->viewable = TRUE;
     impl = GDK_WINDOW_IMPL_DIRECTFB (private->impl);
 
     impl->drawable.wrapper = GDK_DRAWABLE (window);
@@ -2825,6 +2993,7 @@ _gdk_windowing_window_get_next_serial (GdkDisplay *display)
 GdkRegion *
 _gdk_windowing_window_get_input_shape (GdkWindow *window)
 {
+  return NULL;
 }
 
 void
